@@ -1,5 +1,5 @@
 # -*- test-case-name: twisted.trial.test.test_runner -*-
-# Copyright (c) 2001-2010 Twisted Matrix Laboratories.
+# Copyright (c) Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 """
@@ -120,6 +120,20 @@ def _resolveDirectory(fn):
     return fn
 
 
+def _getMethodNameInClass(method):
+    """
+    Find the attribute name on the method's class which refers to the method.
+
+    For some methods, notably decorators which have not had __name__ set correctly:
+
+    getattr(method.im_class, method.__name__) != method
+    """
+    if getattr(method.im_class, method.__name__, object()) != method:
+        for alias in dir(method.im_class):
+            if getattr(method.im_class, alias, object()) == method:
+                return alias
+    return method.__name__
+
 
 class DestructiveTestSuite(TestSuite):
     """
@@ -533,7 +547,7 @@ class TestLoader(object):
         """
         if not isinstance(method, types.MethodType):
             raise TypeError("%r not a method" % (method,))
-        return self._makeCase(method.im_class, method.__name__)
+        return self._makeCase(method.im_class, _getMethodNameInClass(method))
 
     def _makeCase(self, klass, methodName):
         return klass(methodName)
@@ -664,11 +678,27 @@ class TestLoader(object):
             except:
                 errors.append(ErrorHolder(name, failure.Failure()))
         suites = [self.loadAnything(thing, recurse)
-                  for thing in set(things)]
+                  for thing in self._uniqueTests(things)]
         suites.extend(errors)
         return self.suiteFactory(suites)
 
 
+    def _uniqueTests(self, things):
+        """
+        Gather unique suite objects from loaded things. This will guarantee
+        uniqueness of inherited methods on TestCases which would otherwise hash
+        to same value and collapse to one test unexpectedly if using simpler
+        means: e.g. set().
+        """
+        entries = []
+        for thing in things:
+            if isinstance(thing, types.MethodType):
+                entries.append((thing, thing.im_class))
+            else:
+                entries.append((thing,))
+        return [entry[0] for entry in set(entries)]
+
+
 
 class DryRunVisitor(object):
     """
@@ -862,4 +892,3 @@ class TrialRunner(object):
             if not result.wasSuccessful():
                 break
         return result
-
